@@ -45,16 +45,25 @@
 #' ## Link to file - No text to display
 #' writeFormula(wb, "Sheet1", startRow = 4
 #'  , x = makeHyperlinkString(sheet = "testing", row = 3, col = 10
-#'    , file = system.file("loadExample.xlsx", package = "openxlsx")))
+#'    , file = system.file("extdata","loadExample.xlsx", package = "openxlsx")))
 #' 
 #' ## Link to file - Text to display
 #' writeFormula(wb, "Sheet1", startRow = 3
 #'   , x = makeHyperlinkString(sheet = "testing", row = 3, col = 10
-#'     , file = system.file("loadExample.xlsx", package = "openxlsx"), text = "Link to File."))
+#'     , file = system.file("extdata",loadExample.xlsx", package = "openxlsx"), text = "Link to File."))
+#' 
+#' \dontrun{
+#' saveWorkbook(wb, "internalHyperlinks.xlsx",overwrite=TRUE)
+#' }
+#' 
 #' 
-#' saveWorkbook(wb, "internalHyperlinks.xlsx")
 makeHyperlinkString <- function(sheet, row = 1, col = 1, text = NULL, file = NULL){
   
+  od <- getOption("OutDec")
+  options("OutDec" = ".")
+  on.exit(expr = options("OutDec" = od), add = TRUE)
+  
+  
   cell <- paste0(int2col(col), row)
   if(!is.null(file)){
     dest <- sprintf("[%s]'%s'!%s", file, sheet, cell)
@@ -98,7 +107,7 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
     
     ## style hyperlinks
     inds <- which(sapply(colClasses, function(x) "hyperlink" %in% x))
-
+    
     hyperlinkstyle <- createStyle(textDecoration = "underline")
     hyperlinkstyle$fontColour <- list("theme"="10")
     styleElements <- list("style" = hyperlinkstyle,
@@ -172,14 +181,14 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
                           "sheet" =  wb$sheet_names[sheet],
                           "rows" = rep.int(rowInds, times = length(inds)),
                           "cols" = rep(inds + startCol, each = length(rowInds)))
-
+    
     newStylesElements <- append(newStylesElements, list(styleElements))
   }
   
   ## style big mark
   if("scientific" %in% allColClasses){
     inds <- which(sapply(colClasses, function(x) "scientific" %in% x))
-
+    
     styleElements <- list("style" = createStyle(numFmt = "scientific"),
                           "sheet" =  wb$sheet_names[sheet],
                           "rows" = rep.int(rowInds, times = length(inds)),
@@ -191,7 +200,7 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
   ## style big mark
   if("3" %in% allColClasses | "comma" %in% allColClasses){
     inds <- which(sapply(colClasses, function(x) "3" %in% tolower(x) | "comma" %in% tolower(x)))
-
+    
     styleElements <- list("style" = createStyle(numFmt = "3"),
                           "sheet" =  wb$sheet_names[sheet],
                           "rows" = rep.int(rowInds, times = length(inds)),
@@ -230,7 +239,7 @@ classStyles <- function(wb, sheet, startRow, startCol, colNames, nRow, colClasse
     }
     
   }
-    
+  
   
   
   invisible(1)
@@ -302,7 +311,7 @@ writeCommentXML <- function(comment_list, file_name){
   xml <- '<comments xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">'
   xml <- c(xml, paste0('<authors>', paste(sprintf('<author>%s</author>', authors), collapse = ""), '</authors><commentList>'))
   
-
+  
   for(i in 1:length(comment_list)){
     
     authorInd <- which(authors == comment_list[[i]]$author) - 1L
@@ -315,8 +324,8 @@ writeCommentXML <- function(comment_list, file_name){
     
   }
   
-  .Call("openxlsx_writeFile", '', paste(xml, collapse = ""), '</commentList></comments>', file_name, PACKAGE = "openxlsx")
-
+  write_file(body = paste(xml, collapse = ""), tail = '</commentList></comments>', fl = file_name)
+  
   NULL
   
 }
@@ -338,6 +347,12 @@ replaceIllegalCharacters <- function(v){
   v <- gsub('<', "&lt;", v, fixed = TRUE)
   v <- gsub('>', "&gt;", v, fixed = TRUE)
   
+  ## Escape sequences
+  v <- gsub("\a", "", v, fixed = TRUE)
+  v <- gsub("\b", "", v, fixed = TRUE)
+  v <- gsub("\v", "", v, fixed = TRUE)
+  v <- gsub("\f", "", v, fixed = TRUE)
+  
   return(v)
 }
 
@@ -394,7 +409,7 @@ validateBorderStyle <- function(borderStyle){
 getAttrsFont <- function(xml, tag){
   
   
-  x <- lapply(xml, function(x) .Call("openxlsx_getChildlessNode", x, tag, PACKAGE = "openxlsx"))
+  x <- lapply(xml, getChildlessNode, tag = tag)
   x[sapply(x, length) == 0] <- ""
   x <- unlist(x)
   a <- lapply(x, function(x) unlist(regmatches(x, gregexpr('[a-zA-Z]+=".*?"', x))))
@@ -410,7 +425,7 @@ getAttrsFont <- function(xml, tag){
 
 getAttrs <- function(xml, tag){
   
-  x <- lapply(xml, function(x) .Call("openxlsx_getChildlessNode", x, tag, PACKAGE = "openxlsx"))
+  x <- lapply(xml, getChildlessNode, tag = tag)
   x[sapply(x, length) == 0] <- ""
   a <- lapply(x, function(x) regmatches(x, regexpr('[a-zA-Z]+=".*?"', x)))
   
@@ -432,9 +447,9 @@ buildFontList <- function(fonts){
   family <- getAttrs(fonts, "<family ")
   scheme <- getAttrs(fonts, "<scheme ")
   
-  italic <- lapply(fonts, function(x) .Call("openxlsx_getChildlessNode", x, "<i", PACKAGE = "openxlsx"))
-  bold <- lapply(fonts, function(x) .Call("openxlsx_getChildlessNode", x, "<b", PACKAGE = "openxlsx"))
-  underline <- lapply(fonts, function(x) .Call("openxlsx_getChildlessNode", x, "<u", PACKAGE = "openxlsx"))
+  italic <- lapply(fonts, getChildlessNode, tag = "<i")
+  bold <- lapply(fonts, getChildlessNode, tag = "<b")
+  underline <- lapply(fonts, getChildlessNode, tag = "<u")
   
   ## Build font objects
   ft <- replicate(list(), n=length(fonts))
@@ -501,15 +516,20 @@ get_named_regions_from_string <- function(dn){
   dn <- gsub("</workbook>", "", dn, fixed = TRUE)
   
   dn <- unique(unlist(strsplit(dn, split = "</definedName>", fixed = TRUE)))
+  dn <- dn[grepl("<definedName", dn, fixed=TRUE)]
   
   dn_names <- regmatches(dn, regexpr('(?<=name=")[^"]+', dn, perl = TRUE))
   
   dn_pos <- regmatches(dn, regexpr("(?<=>).*", dn, perl = TRUE))
-  dn_coords <- regmatches(dn_pos, regexpr("(?<=!).*", dn_pos, perl = TRUE))
-  dn_coords <- gsub("$", "", dn_coords, fixed = TRUE)
+  dn_pos <- gsub("[$']", "", dn_pos)
   
-  dn_sheets <- regmatches(dn_pos, regexpr(".*(?=!)", dn_pos, perl = TRUE))
-  dn_sheets <- gsub("'", "", dn_sheets, fixed = TRUE)
+  has_bang <- grepl("!", dn_pos, fixed=TRUE)
+  dn_sheets <- ifelse(has_bang,
+                      gsub("^(.*)!.*$", "\\1", dn_pos),
+                      "")
+  dn_coords <- ifelse(has_bang,
+                      gsub("^.*!(.*)$", "\\1", dn_pos),
+                      "")
   
   attr(dn_names, "sheet") <- dn_sheets
   attr(dn_names, "position") <- dn_coords
@@ -551,20 +571,27 @@ nodeAttributes <- function(x){
 
 buildBorder <- function(x){
   
+  style <- list()
+  if(grepl('diagonalup="1"', tolower(x), fixed = TRUE))
+    style$borderDiagonalUp <- TRUE
+  
+  if(grepl('diagonaldown="1"', tolower(x), fixed = TRUE))
+    style$borderDiagonalDown <- TRUE
+  
   ## gets all borders that have children
-  x <- unlist(lapply(c("<left", "<right", "<top", "<bottom"), function(tag) .Call("openxlsx_getNodes", x, tag, PACKAGE = "openxlsx")))
+  x <- unlist(lapply(c("<left", "<right", "<top", "<bottom", "<diagonal"), function(tag) getNodes(xml = x, tagIn = tag)))
   if(length(x) == 0)
     return(NULL)
   
-  sides <- c("TOP", "BOTTOM", "LEFT", "RIGHT")
-  sideBorder <- character(length=length(x))
+  sides <- c("TOP", "BOTTOM", "LEFT", "RIGHT", "DIAGONAL")
+  sideBorder <- character(length = length(x))
   for(i in 1:length(x)){
     tmp <- sides[sapply(sides, function(s) grepl(s, x[[i]], ignore.case = TRUE))]
     if(length(tmp) > 1) tmp <- tmp[[1]]
     if(length(tmp) == 1)
       sideBorder[[i]] <- tmp
   }
-
+  
   sideBorder <- sideBorder[sideBorder != ""]
   x <- x[sideBorder != ""]
   if(length(sideBorder) == 0)
@@ -577,7 +604,7 @@ buildBorder <- function(x){
   
   ## Colours
   cols <- replicate(n = length(sideBorder), list(rgb = "FF000000"))
-  colNodes <- unlist(sapply(x, function(xml) .Call("openxlsx_getChildlessNode", xml, "<color", PACKAGE = "openxlsx"), USE.NAMES = FALSE))
+  colNodes <- unlist(sapply(x, getChildlessNode, tag = "<color", USE.NAMES = FALSE))
   
   if(length(colNodes) > 0){
     attrs <- regmatches(colNodes, regexpr('(theme|indexed|rgb|auto)=".+"', colNodes))
@@ -607,8 +634,6 @@ buildBorder <- function(x){
   })
   
   ## sideBorder & cols
-  style <- list()
-  
   if("LEFT" %in% sideBorder){
     style$borderLeft <- weight[which(sideBorder == "LEFT")]
     style$borderLeftColour <- cols[which(sideBorder == "LEFT")]
@@ -629,6 +654,11 @@ buildBorder <- function(x){
     style$borderBottomColour <- cols[which(sideBorder == "BOTTOM")]
   }
   
+  if("DIAGONAL" %in% sideBorder){
+    style$borderDiagonal <- weight[which(sideBorder == "DIAGONAL")]
+    style$borderDiagonalColour <- cols[which(sideBorder == "DIAGONAL")]
+  }
+  
   return(style)
 }
 
@@ -706,7 +736,7 @@ buildFillList <- function(fills){
   
   ## gradientFill
   inds <- grepl("gradientFill", fills)
-  fillAttrs[inds] <- fills[inds] #lapply(fills[inds], function(x) .Call("openxlsx_getNodes", x, "<gradientFill>", PACKAGE = "openxlsx"))
+  fillAttrs[inds] <- fills[inds]
   
   return(fillAttrs)
   
@@ -730,7 +760,7 @@ getDefinedNamesSheet <- function(x){
 getSharedStringsFromFile <- function(sharedStringsFile, isFile){
   
   ## read in, get si tags, get t tag value and  pull out all string nodes
-  sharedStrings = .Call("openxlsx_get_shared_strings", sharedStringsFile, isFile, PACKAGE = 'openxlsx') ## read from file
+  sharedStrings = get_shared_strings(xmlFile = sharedStringsFile, isFile = isFile) ## read from file
   
   
   Encoding(sharedStrings) <- "UTF-8"
@@ -747,9 +777,9 @@ getSharedStringsFromFile <- function(sharedStringsFile, isFile){
 }
 
 
-clean_names <- function(x){
+clean_names <- function(x,schar){
   x <- gsub("^[[:space:]]+|[[:space:]]+$", "", x)
-  x <- gsub("[[:space:]]+", ".", x)
+  x <- gsub("[[:space:]]+", schar, x)
   return(x)
 }
 
@@ -760,7 +790,7 @@ mergeCell2mapping <- function(x){
   refs <- regmatches(x, regexpr("(?<=ref=\")[A-Z0-9:]+", x, perl = TRUE))
   refs <- strsplit(refs, split = ":")
   rows <- lapply(refs, function(r) {
-    r <- as.integer(gsub("[A-Z]", "", r))
+    r <- as.integer(gsub(pattern = "[A-Z]", replacement = "", r, perl = TRUE))
     seq(from = r[1], to = r[2], by = 1)
   })
   
@@ -772,7 +802,7 @@ mergeCell2mapping <- function(x){
   ## for each we grid.expand
   refs <- do.call("rbind", lapply(1:length(rows), function(i){
     tmp <- expand.grid("cols" = cols[[i]], "rows" = rows[[i]])
-    tmp$ref <- paste0(.Call("openxlsx_convert_to_excel_ref", tmp$cols, LETTERS, PACKAGE = "openxlsx"), tmp$rows)
+    tmp$ref <- paste0(convert_to_excel_ref(cols = tmp$cols, LETTERS = LETTERS), tmp$rows)
     tmp$anchor_cell <- tmp$ref[1]
     return(tmp[, c("anchor_cell", "ref", "rows")])
   }))
@@ -840,3 +870,23 @@ getFile <- function(xlsxFile){
   
 }
 
+# Rotate the 15-bit integer by n bits to the 
+hashPassword <- function(password) {
+  # password limited to 15 characters
+  chars = head(strsplit(password, "")[[1]], 15)
+  # See OpenOffice's documentation of the Excel format: http://www.openoffice.org/sc/excelfileformat.pdf
+  # Start from the last character and for each character
+  # - XOR hash with the ASCII character code
+  # - rotate hash (16 bits) one bit to the left
+  # Finally, XOR hash with 0xCE4B and XOR with password length
+  # Output as hex (uppercase)
+  rotate16bit <- function(hash, n = 1) {
+    bitwOr(bitwAnd(bitwShiftR(hash, 15 - n), 0x01), bitwAnd(bitwShiftL(hash, n), 0x7fff));
+  }
+  hash = Reduce(function(char, h) {
+    h = bitwXor(h, as.integer(charToRaw(char)))
+    rotate16bit(h, 1)
+  }, chars, 0, right = TRUE)
+  hash = bitwXor(bitwXor(hash, length(chars)), 0xCE4B)
+  format(as.hexmode(hash), upper.case = TRUE)
+}
\ No newline at end of file
