@@ -7,7 +7,7 @@ context("Load Workbook Object")
 
 test_that("Loading readTest.xlsx Sheet 1", {
   
-  fl <- system.file("readTest.xlsx", package = "openxlsx")
+  fl <- system.file("extdata","readTest.xlsx", package = "openxlsx")
   wb <- loadWorkbook(fl)
   
   sheet_data <- wb$worksheets[[2]]$sheet_data
@@ -109,7 +109,7 @@ test_that("Loading readTest.xlsx Sheet 1", {
 
 test_that("Loading readTest.xlsx Sheet 1", {
   
-  fl <- system.file("readTest.xlsx", package = "openxlsx")
+  fl <- system.file("extdata","readTest.xlsx", package = "openxlsx")
   wb <- loadWorkbook(fl)
   
   sheet_data <- wb$worksheets[[1]]$sheet_data
@@ -880,8 +880,8 @@ test_that("Loading readTest.xlsx Sheet 1", {
                                          "<si><t>a</t></si>", "<si><t>b</t></si>", "<si><t>c</t></si>", 
                                          "<si><t>e</t></si>", "<si><t>f</t></si>", "<si><t>h</t></si>", 
                                          "<si><t>i</t></si>", "<si><t>Var5</t></si>", "<si><t>Var6</t></si>", 
-                                         "<si><t>Var7</t></si>", "<si><t>col 1</t></si>", "<si><t>col 2</t></si>", "<si><t>g</t></si>"
-  ), uniqueCount = 2113L)
+                                         "<si><t>Var7</t></si>", "<si><t>col 1</t></si>", "<si><t>col 2</t></si>", "<si><t>g</t></si>", "<si><t>x</t></si>"
+  ), uniqueCount = 2114L)
   
   
   expect_equal(expected_shared_strings, wb$sharedStrings)
