@@ -1,7 +1,6 @@
 
 
 
-
 context("Named Regions")
 
 
@@ -64,7 +63,7 @@ test_that("Correctly Loading Named Regions Created in Excel",{
   # Load an excel workbook (in the repo, it's located in the /inst folder;
   # when installed on the user's system, it is located in the installation folder
   # of the package)
-  filename <- system.file("namedRegions.xlsx", package = "openxlsx")
+  filename <- system.file("extdata","namedRegions.xlsx", package = "openxlsx")
   
   # Load this workbook. We will test read.xlsx by passing both the object wb and
   # the filename. Both should produce the same results.
@@ -118,6 +117,25 @@ test_that("Correctly Loading Named Regions Created in Excel",{
 })
 
 
+test_that("Load names from an Excel file with funky non-region names", {
+  filename <- system.file("extdata","namedRegions2.xlsx", package = "openxlsx")
+  wb <- loadWorkbook(filename)
+  names <- getNamedRegions(wb)
+  sheets <- attr(names, "sheet")
+  positions <- attr(names, "position")
+  
+  expect_true(length(names) == length(sheets))
+  expect_true(length(names) == length(positions))
+  expect_equal(head(names, 5),
+               c("barref", "barref", "fooref", "fooref", "IQ_CH"))
+  expect_equal(sheets,
+               c("Sheet with space", "Sheet1", "Sheet with space", "Sheet1",
+                 rep("", 26)))
+  expect_equal(positions, c("B4", "B4", "B3", "B3", rep("", 26)))
+  
+  names2 <- getNamedRegions(filename)
+  expect_equal(names, names2)
+})
 
 
 test_that("Missing rows in named regions", {
